/* config.cc: Configuration handling implementation.
 *
 * Copyright (c) 2006 Jonne Lehtinen <jms@modeemi.fi>
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the Do What The F*ck You Want To
 *  Public License, Version 2, as published by Sam Hocevar. See
 *  COPYING file for more details.
 */

#include <xmmsclient/xmmsclient++.h>
#include "config.hh"

#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"

#include <iostream>
#include <sstream>

#include <signal.h>

namespace Xmms {

    inline std::string getUserConfDir() {

        char buf[PATH_MAX] = { '\0' };
        if( !xmmsc_userconfdir_get( buf, PATH_MAX ) ) {
            throw Xmms::result_error( "Error occured when trying to get "
                                      "user config directory." );
        }
        return std::string( buf );

    }

}

gx2osd::config::config() 
    : noncopyable(),
      config_dir_( Xmms::getUserConfDir() + "/clients/gx2osd", fs::native ),
      config_file_( config_dir_ / "gx2osd.conf" ),
      pid_file_( config_dir_ / "gx2osd.pid" ),
      desc_( "gx2osd options" ),
      vm_(),
      skin_(),
      xpos_( 0 ),
      ypos_( 0 ),
      centered_( false ) {

    if( !fs::exists( config_dir_ ) ) {
        fs::create_directory( config_dir_ );
    }

    desc_.add_options()
        ( "help,h", "Print this message." )
        ( "show", "Show already running OSD window." )
        ( "hide-delay,d",
          po::value< unsigned int >( &delay_ )->default_value( 3000 ),
          "Time to wait in ms before hiding the OSD window." )
        ( "position,p", 
          po::value< std::string >()->default_value("0 x 0")->notifier( 
              boost::bind(&gx2osd::config::parse_position, this, _1) ),
          "Main window position, either \"X x Y\" or \"center\"." )
        ( "skin,s",
          po::value< std::string >( &skin_name_ )->default_value( "default" ),
          "skin to use." );

}

gx2osd::config::~config() {

}

gx2osd::config&
gx2osd::config::get_instance() {
    static config instance;
    return instance;
}

bool
gx2osd::config::set_args( const cmd_line& args ) {

    std::cout << "Parsing arguments and config file..." << std::endl;
    po::store( po::command_line_parser( args ).options( desc_ ).run(), vm_ );

    if( fs::exists( config_file_ ) ) {

        fs::ifstream ifs( config_file_ );
        po::store( po::parse_config_file( ifs, desc_ ), vm_ );

    }

    po::notify( vm_ );
    if( vm_.count( "help" ) ) {
        std::cout << desc_ << std::endl;
        return false;
    }
    if( vm_.count( "show" ) ) {

        fs::ifstream ifs( pid_file_ );
        pid_t pid;
        ifs >> pid;
        kill( pid, SIGUSR1 );
        return false;

    }

    std::cout << "Loading skin: " << skin_name_ << std::endl;
    skin_.reset( new skin( skin_name_ ) );

    return true;

}

bool
gx2osd::config::get_centered() const {
    return centered_;
}

unsigned int
gx2osd::config::get_xpos() const {
    return xpos_;
}

unsigned int
gx2osd::config::get_ypos() const {
    return ypos_;
}

unsigned int
gx2osd::config::get_delay() const {
    return delay_;
}

fs::path
gx2osd::config::get_config_dir() const {
    return config_dir_;
}

const fs::path&
gx2osd::config::get_pid_file() const {
    return pid_file_;
}

const boost::shared_ptr< gx2osd::skin >&
gx2osd::config::get_skin() const {
    return skin_;
}

void
gx2osd::config::parse_position( const std::string& position ) {

    if( position == "center" ) {
        centered_ = true;
    }
    else {
        std::istringstream ss( position );
        char temp;
        if( !(ss >> xpos_ >> temp >> ypos_ && temp == 'x') ) {
            std::cerr << "Invalid position " << position << ". "
                      << "Setting position to 0 x 0."
                      << std::endl;
            xpos_ = 0; ypos_ = 0;
        }
    }

}

gx2osd::config&
gx2osd::get_config() {
    return gx2osd::config::get_instance();
}
