/* main_window.cc: OSD Window implementation.
 *
 * Copyright (c) 2006 Jonne Lehtinen <jms@modeemi.fi>
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the Do What The F*ck You Want To
 *  Public License, Version 2, as published by Sam Hocevar. See
 *  COPYING file for more details.
 */

#include <gtk/gtk.h>
#include <gtkmm/window.h>
#include <gtkmm/label.h>
#include <gdkmm/screen.h>
#include <gdkmm/pixbuf.h>
#include <gdkmm/pixbufloader.h>
#include <gtkmm/image.h>
#include <glibmm/ustring.h>

#include "main_window.hh"
#include "config.hh"

#include <iostream>

#include <xmmsclient/xmmsclient++.h>


gx2osd::main_window::main_window() 
    : Gtk::Window( Gtk::WINDOW_POPUP ),
      artist_name_(),
      song_name_(),
      fixed_pane_(),
      artist_markup_(),
      song_name_markup_(),
      span_end_( "</span>" ),
      cover_(),
      status_(),
      skin_( get_config().get_skin() ) {

    // TODO: find a way to do this in gtkmm... or prove not possible yet.
    gtk_window_set_accept_focus( GTK_WINDOW( gobj() ), FALSE );
    set_decorated( false );
    set_keep_above( true );
    if( get_config().get_centered() ) {
        set_position(Gtk::WIN_POS_CENTER);
    }
    else {
        move( get_config().get_xpos(), get_config().get_ypos() );
    }

    fixed_pane_.add( background_ );

    load_skin();

    fixed_pane_.put( artist_name_, skin_->get_artist().get_position().x,
                                   skin_->get_artist().get_position().y );
    fixed_pane_.put( song_name_, skin_->get_songname().get_position().x,
                                 skin_->get_songname().get_position().y );
    fixed_pane_.put( cover_, skin_->no_cover_image().get_position().x, 
                             skin_->no_cover_image().get_position().y );
    fixed_pane_.put( status_, skin_->get_status_position().x, 
                              skin_->get_status_position().y );

    add( fixed_pane_ );

}

gx2osd::main_window::~main_window() {

}

gx2osd::main_window&
gx2osd::main_window::get_instance() {
    static main_window instance;
    return instance;
}

void
gx2osd::main_window::set_text( const Glib::ustring& artist,
                               const Glib::ustring& title,
                               const Glib::ustring& album ) {

    artist_name_.set_markup( artist_markup_ 
                           + Glib::Markup::escape_text( artist )
                           + span_end_ );

    song_name_.set_markup( song_name_markup_
                         + Glib::Markup::escape_text( title )
                         + span_end_ );

    if( skin_->has_album() ) {
        album_name_.set_markup( album_name_markup_
                              + Glib::Markup::escape_text( album )
                              + span_end_ );
    }

}

void
gx2osd::main_window::set_text_and_show( const Glib::ustring& artist,
                                        const Glib::ustring& title,
                                        const Glib::ustring& album ) {

    cover_.set( skin_->no_cover_image() );
    set_text( artist, title, album );
    show_all();

}

void
gx2osd::main_window::set_cover( const Xmms::bin& cover,
                                const Glib::ustring& artist,
                                const Glib::ustring& title,
                                const Glib::ustring& album ) {

    try {

        Glib::RefPtr<Gdk::PixbufLoader> loader = Gdk::PixbufLoader::create();

        loader->write( cover.data(), cover.size() );
        loader->close();

        cover_.set( loader->get_pixbuf()->scale_simple( 100, 100, Gdk::INTERP_BILINEAR ) );

    }
    catch ( Glib::Error& e ) {
        std::cout << "Could not load image: " << e.what() << std::endl;
        cover_.set( skin_->no_cover_image() );
    }

    set_text( artist, title, album );
    show_all();

}

bool
gx2osd::main_window::set_status( const Xmms::Playback::Status& status ) {
    set_status_no_show( status );
    show_all();
    return true;
}

void
gx2osd::main_window::set_status_no_show( const Xmms::Playback::Status& status ) {
    switch( status ) {
        case Xmms::Playback::STOPPED: {
            status_.set( skin_->stopped_image() );
            break;
        }
        case Xmms::Playback::PLAYING: {
            status_.set( skin_->playing_image() );
            break;
        }
        case Xmms::Playback::PAUSED: {
            status_.set( skin_->paused_image() );
            break;
        }
    }
}

void
gx2osd::main_window::show_all() {
    static sigc::connection conn;
    Gtk::Window::show_all();
    if( conn ) {
        conn.disconnect();
    }
    conn = Glib::signal_timeout().connect( sigc::mem_fun(*this,
                                           &main_window::hider),
                                           get_config().get_delay() );
}

bool
gx2osd::main_window::hider() {
    hide();
    return false;
}

void
gx2osd::main_window::load_skin() {

    set_label_stuff( artist_name_, skin_->get_artist(), artist_markup_ );
    set_label_stuff( song_name_, skin_->get_songname(), song_name_markup_ );

    if( skin_->has_album() ) {

        set_label_stuff( album_name_, skin_->get_album(), album_name_markup_ );
        fixed_pane_.put( album_name_, skin_->get_album().get_position().x,
                                      skin_->get_album().get_position().y );

    }
    else if( album_name_.get_parent() != 0 ) {
        fixed_pane_.remove( album_name_ );
    }

    Glib::RefPtr<Gdk::Pixbuf> buf( skin_->background() );

    if( buf ) {

        Glib::RefPtr<Gdk::Bitmap> mask;
        Glib::RefPtr<Gdk::Pixmap> pixmap;

        buf->render_pixmap_and_mask( pixmap, mask, 128 );

        resize( buf->get_width(), buf->get_height() );
        shape_combine_mask(mask, 0, 0);

        background_.set( buf );

    }
    
    cover_.set( skin_->no_cover_image() );
    status_.set( skin_->paused_image() );
    
}

void
gx2osd::main_window::set_label_stuff( Gtk::Label& gtklabel,
                                      const gx2osd::label& label,
                                      Glib::ustring& markup ) {

    markup = "<span weight=\"";
    gtklabel.set_width_chars( label.get_width() );

    if( label.get_bold() ) {
        markup += "bold\"";
    }
    else {
        markup += "normal\"";
    }
    markup += " color=\"" + label.get_color() + "\">";

    Pango::FontDescription font;
    font.set_size( label.get_size() * Pango::SCALE );
    gtklabel.modify_font( font );

}

gx2osd::main_window&
gx2osd::get_window() {
    return gx2osd::main_window::get_instance();
}
