/* skin.cc: skin format implementation.
 *
 * Copyright (c) 2006 Jonne Lehtinen <jms@modeemi.fi>
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the Do What The F*ck You Want To
 *  Public License, Version 2, as published by Sam Hocevar. See
 *  COPYING file for more details.
 */

#include "skin.hh"
#include "config.hh"
#include "image.hh"
#include "label.hh"

#include <gdkmm/pixbuf.h>

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>

#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/dom/DOMNode.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#include <xercesc/dom/DOMElement.hpp>

#include <xercesc/sax/SAXParseException.hpp>
#include <xercesc/sax/ErrorHandler.hpp>

#include "boost/filesystem/path.hpp"
#include "boost/filesystem/operations.hpp"

#include <string>
#include <sstream>
#include <iostream>

std::ostream& operator<<( std::ostream& stream, const XMLCh* const c ) {
    char* temp = xercesc::XMLString::transcode( c );
    stream << temp;
    xercesc::XMLString::release( &temp );
    return stream;
}

class err_handler : public xercesc::ErrorHandler {

    public:
        err_handler() {}
        virtual ~err_handler() {}

        void warning( const xercesc::SAXParseException& e ) {
            std::cerr << "Warning (" << e.getLineNumber() << ":"
                      << e.getColumnNumber() << "): "
                      << e.getMessage() << std::endl;
        }

        void error( const xercesc::SAXParseException& e ) {
            std::ostringstream ss;
            ss << "Error (" << e.getLineNumber() << ":"
               << e.getColumnNumber() << "): "
               << e.getMessage();
            throw std::runtime_error( ss.str() );
        }

        void fatalError( const xercesc::SAXParseException& e ) {
            std::ostringstream ss;
            ss << "Fatal error (" << e.getLineNumber() << ":"
               << e.getColumnNumber() << "): "
               << e.getMessage();
            throw std::runtime_error( ss.str() );
        }

        void resetErrors() {}

};

class XMLChDeleter {

    public:
        XMLChDeleter() {}
        ~XMLChDeleter() {}

        void operator()( XMLCh*& p ) {
            xercesc::XMLString::release( &p );
        }

};

gx2osd::skin::skin( const std::string& name )
    : noncopyable(),
      name_( name ),
      skin_path_( get_config().get_config_dir() / "skins" ),
      background_(),
      no_cover_image_(),
      paused_image_(), 
      stopped_image_(), 
      playing_image_(),
      artist_(), 
      album_(),
      songname_(), 
      has_album_( false ),
      status_position_() {

    if( !fs::exists( (skin_path_ / name) / (name + ".xml") ) ) {
#ifdef PREFIX
        skin_path_ = PREFIX "/share/gx2osd/skins";
        if( !fs::exists( (skin_path_ / name) / (name + ".xml") ) ) {
#endif
        throw std::invalid_argument( "Could not find skin." );
#ifdef PREFIX
        }
#endif
    }

    xercesc::XMLPlatformUtils::Initialize();

    boost::shared_ptr< xercesc::XercesDOMParser > parser_;

    err_handler err;
    parser_.reset( new xercesc::XercesDOMParser() );
    parser_->setErrorHandler( &err );
    parser_->setValidationScheme( xercesc::XercesDOMParser::Val_Always );
    parser_->setValidationSchemaFullChecking( true );
    parser_->setDoNamespaces( true );
    parser_->setDoSchema( true );
#ifdef PREFIX
    parser_->setExternalNoNamespaceSchemaLocation( 
            (fs::path(PREFIX) / "share/gx2osd/skins/skin.xsd").string().c_str() );
#endif

    parser_->parse( ((skin_path_ / name) / (name + ".xml")).string().c_str() );

    xercesc::DOMDocument* doc = parser_->getDocument();

    using xercesc::XMLString;

    xmlch background_tag( XMLString::transcode("background"), XMLChDeleter() );
    xmlch cover_tag( XMLString::transcode("cover"), XMLChDeleter() );
    xmlch paused_tag( XMLString::transcode("paused"), XMLChDeleter() );
    xmlch stopped_tag( XMLString::transcode("stopped"), XMLChDeleter() );
    xmlch playing_tag( XMLString::transcode("playing"), XMLChDeleter() );

    load_image( background_, background_tag, doc );
    load_image( no_cover_image_, cover_tag, doc );
    load_image( paused_image_, paused_tag, doc );
    load_image( stopped_image_, stopped_tag, doc );
    load_image( playing_image_, playing_tag, doc );

    xmlch artist_tag( XMLString::transcode("artist"), XMLChDeleter() );
    xmlch album_tag( XMLString::transcode("album"), XMLChDeleter() );
    xmlch songname_tag( XMLString::transcode("songname"), XMLChDeleter() );

    load_label( artist_, artist_tag, doc );
    has_album_ = load_label( album_, album_tag, doc );
    load_label( songname_, songname_tag, doc );

    xmlch status_tag( XMLString::transcode("status"), XMLChDeleter() );

    using xercesc::DOMElement;

    DOMElement* el = 0;
    if( (el = find_element( status_tag, doc )) ) {
        parse_position( el, status_position_.x, status_position_.y );
    }

    parser_.reset();
    xercesc::XMLPlatformUtils::Terminate();

}

gx2osd::skin::~skin() {

}

gx2osd::image&
gx2osd::skin::background() {
    return background_;
}

gx2osd::image&
gx2osd::skin::no_cover_image() {
    return no_cover_image_;
}

gx2osd::image&
gx2osd::skin::paused_image() {
    return paused_image_;
}

gx2osd::image&
gx2osd::skin::stopped_image() {
    return stopped_image_;
}

gx2osd::image&
gx2osd::skin::playing_image() {
    return playing_image_;
}

const gx2osd::Position&
gx2osd::skin::get_status_position() const {
    return status_position_;
}

const gx2osd::label&
gx2osd::skin::get_artist() const {
    return artist_;
}

const gx2osd::label&
gx2osd::skin::get_album() const {
    return album_;
}

const gx2osd::label&
gx2osd::skin::get_songname() const {
    return songname_;
}

bool
gx2osd::skin::has_album() const {
    return has_album_;
}

void
gx2osd::skin::load_image( gx2osd::image& i,
                          boost::shared_ptr< XMLCh > tag_name,
                          xercesc::DOMDocument*& doc ) {

    using xercesc::XMLString;
    using xercesc::DOMElement;

    xmlch image_attr( XMLString::transcode("image"), XMLChDeleter() );

    DOMElement* el = 0;
    if( (el = find_element( tag_name, doc )) ) {

        std::stringstream ss;
        ss << el->getAttribute( &*image_attr );

        std::clog << "\tLoading image: " << ss.str() << std::endl;
        fs::path image_file( skin_path_ / name_ / ss.str() );
        if( !exists( image_file ) ) {
            throw std::invalid_argument( std::string( "Could not find image: " )
                    + image_file.string() );
        }

        unsigned int x = 0;
        unsigned int y = 0;
        parse_position( el, x, y );

        i.set_position( x, y );

        i = Gdk::Pixbuf::create_from_file( image_file.string() );

    }

}

bool gx2osd::skin::load_label( label& l,
                               boost::shared_ptr< XMLCh > tag_name,
                               xercesc::DOMDocument*& doc ) {

    using xercesc::XMLString;
    using xercesc::DOMElement;

    DOMElement* el = 0;
    if( (el = find_element( tag_name, doc )) ) {

        unsigned int x = 0, y = 0;
        parse_position( el, x, y );
        l.set_position( x, y );

        xmlch width_attr( XMLString::transcode("width"), XMLChDeleter() );

        std::stringstream ss;
        ss << el->getAttribute( &*width_attr );

        unsigned int width = 0;
        ss >> width;

        l.set_width( width );

        xmlch font_elem( XMLString::transcode("font"), XMLChDeleter() );
        xercesc::DOMNodeList* nodes = el->getElementsByTagName( &*font_elem );
        if( nodes->getLength() > 0 ) {

            el = dynamic_cast< DOMElement* >( nodes->item( 0 ) );

            std::cout << "\tLoading font attributes... " << std::flush;

            xmlch size_attr( XMLString::transcode("size"), XMLChDeleter() );
            xmlch bold_attr( XMLString::transcode("bold"), XMLChDeleter() );
            xmlch color_attr( XMLString::transcode("color"), XMLChDeleter() );

            if( el->hasAttribute( &*size_attr ) ) {
                std::cout << "size = " << std::flush;
                ss.str("");
                ss.clear();
                ss << el->getAttribute( &*size_attr );
                unsigned int size = 0;
                ss >> size;
                l.set_size( size );
                std::cout << size << "... " << std::flush;
            }
            if( el->hasAttribute( &*bold_attr ) ) {
                std::cout << "bold = " << std::flush;
                ss.str("");
                ss.clear();
                ss << el->getAttribute( &*bold_attr );
                l.set_bold( ss.str() == "true" );
                std::cout << ss.str() << "... " << std::flush;
            }
            if( el->hasAttribute( &*color_attr ) ) {
                std::cout << "color = " << std::flush;
                ss.str("");
                ss.clear();
                ss << el->getAttribute( &*color_attr );
                l.set_color( ss.str() );
                std::cout << ss.str() << "...";
            }
            std::cout << std::endl;

        }

        return true;

    }
    else {
        return false;
    }

}

void
gx2osd::skin::parse_position( xercesc::DOMElement*& element,
                              unsigned int& x, unsigned int& y ) {

    using xercesc::XMLString;

    xmlch posit_attr( XMLString::transcode("position"), XMLChDeleter() );

    if( element->hasAttribute( &*posit_attr ) ) {

        std::stringstream ss;
        ss << element->getAttribute( &*posit_attr );

        char temp = 'x';
        ss >> x >> temp >> y;

    }

}

xercesc::DOMElement*
gx2osd::skin::find_element( xmlch tag_name, xercesc::DOMDocument*& doc ) {

    using xercesc::DOMNodeList;
    using xercesc::DOMNode;
    using xercesc::DOMElement;

    DOMElement* el = 0;

    DOMNodeList* nodes = doc->getElementsByTagName( &*tag_name );
    if( nodes->getLength() > 0 ) {

        DOMNode* node = nodes->item( 0 );
        if( node->getNodeType() == DOMNode::ELEMENT_NODE ) {

            el = dynamic_cast< DOMElement* >( node );

        }

    }

    return el;

}
